"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const error_1 = require("../error");
const logger = require("../logger");
function getRulesConfig(projectId, options) {
    const config = options.config;
    const dbConfig = config.get("database");
    if (dbConfig === undefined) {
        return [];
    }
    if (!Array.isArray(dbConfig)) {
        if (dbConfig && dbConfig.rules) {
            return [{ rules: dbConfig.rules, instance: options.instance }];
        }
        else {
            logger.debug("Possibly invalid database config: ", JSON.stringify(dbConfig));
            return [];
        }
    }
    const results = [];
    for (const c of dbConfig) {
        if (c.target) {
            options.rc.requireTarget(projectId, "database", c.target);
            const instances = options.rc.target(projectId, "database", c.target);
            for (const i of instances) {
                results.push({ instance: i, rules: c.rules });
            }
        }
        else if (c.instance) {
            results.push(c);
        }
        else {
            throw new error_1.FirebaseError('Must supply either "target" or "instance" in database config');
        }
    }
    return results;
}
exports.getRulesConfig = getRulesConfig;
