"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const logger = require("../../../logger");
const apiEnabled = require("../../../ensureApiEnabled");
const ensureCloudResourceLocation_1 = require("../../../ensureCloudResourceLocation");
const requirePermissions_1 = require("../../../requirePermissions");
const checkDatabaseType_1 = require("../../../firestore/checkDatabaseType");
const rules = require("./rules");
const indexes = require("./indexes");
const error_1 = require("../../../error");
const clc = require("cli-color");
function doSetup(setup, config) {
    return __awaiter(this, void 0, void 0, function* () {
        const firestoreUnusedError = new error_1.FirebaseError(`It looks like you haven't used Cloud Firestore in this project before. Go to ${clc.bold.underline(`https://console.firebase.google.com/project/${setup.projectId}/database`)} to create your Cloud Firestore database.`, { exit: 1 });
        const isFirestoreEnabled = yield apiEnabled.check(setup.projectId, "firestore.googleapis.com", "", true);
        if (!isFirestoreEnabled) {
            throw firestoreUnusedError;
        }
        const dbType = yield checkDatabaseType_1.checkDatabaseType(setup.projectId);
        logger.debug(`database_type: ${dbType}`);
        if (!dbType) {
            throw firestoreUnusedError;
        }
        else if (dbType !== "CLOUD_FIRESTORE") {
            throw new error_1.FirebaseError(`It looks like this project is using Cloud Datastore or Cloud Firestore in Datastore mode. The Firebase CLI can only manage projects using Cloud Firestore in Native mode. For more information, visit https://cloud.google.com/datastore/docs/firestore-or-datastore`, { exit: 1 });
        }
        setup.config.firestore = {};
        ensureCloudResourceLocation_1.ensureLocationSet(setup.projectLocation, "Cloud Firestore");
        yield requirePermissions_1.requirePermissions({ project: setup.projectId });
        yield rules.initRules(setup, config);
        yield indexes.initIndexes(setup, config);
    });
}
exports.doSetup = doSetup;
