"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const google_auth_library_1 = require("google-auth-library");
const clc = require("cli-color");
const api = require("./api");
const configstore_1 = require("./configstore");
const error_1 = require("./error");
const logger = require("./logger");
const utils = require("./utils");
const scopes = require("./scopes");
const AUTH_ERROR_MESSAGE = `Command requires authentication, please run ${clc.bold("firebase login")}`;
let authClient;
function getAuthClient(config) {
    if (authClient) {
        return authClient;
    }
    authClient = new google_auth_library_1.GoogleAuth(config);
    return authClient;
}
function autoAuth(options, authScopes) {
    return __awaiter(this, void 0, void 0, function* () {
        const client = getAuthClient({ scopes: authScopes, projectId: options.project });
        const token = yield client.getAccessToken();
        api.setAccessToken(token);
    });
}
function requireAuth(options) {
    return __awaiter(this, void 0, void 0, function* () {
        api.setScopes([scopes.CLOUD_PLATFORM, scopes.FIREBASE_PLATFORM]);
        options.authScopes = api.getScopes();
        const tokens = configstore_1.configstore.get("tokens");
        const user = configstore_1.configstore.get("user");
        let tokenOpt = utils.getInheritedOption(options, "token");
        if (tokenOpt) {
            logger.debug("> authorizing via --token option");
        }
        else if (process.env.FIREBASE_TOKEN) {
            logger.debug("> authorizing via FIREBASE_TOKEN environment variable");
        }
        else if (user) {
            logger.debug("> authorizing via signed-in user");
        }
        else {
            try {
                return yield autoAuth(options, options.authScopes);
            }
            catch (e) {
                throw new error_1.FirebaseError(`Failed to authenticate, have you run ${clc.bold("firebase login")}?`, { original: e });
            }
        }
        tokenOpt = tokenOpt || process.env.FIREBASE_TOKEN;
        if (tokenOpt) {
            api.setRefreshToken(tokenOpt);
            return;
        }
        if (!user || !tokens) {
            throw new error_1.FirebaseError(AUTH_ERROR_MESSAGE);
        }
        options.user = user;
        options.tokens = tokens;
        api.setRefreshToken(tokens.refresh_token);
    });
}
exports.requireAuth = requireAuth;
